# Efficient-3DCNNs
PyTorch Implementation of the article "[DriverMHG: A Multi-Modal Dataset for Dynamic Recognition of Driver Micro Hand Gestures and a Real-Time Recognition Framework](https://arxiv.org/pdf/2003.00951.pdf)".

## Requirements

* [PyTorch > 1.0]
* OpenCV
* FFmpeg, FFprobe
* Python 3

## Used models

Implemented models:
 - 3D SqueezeNet
 - 3D MobileNet
 - 3D ShuffleNet
 - 3D MobileNetv2
 - 3D ShuffleNetv2
 
 All models (except for SqueezeNet) are evaluated for 4 different complexity levels by adjusting their 'width_multiplier' with 2 different hardware platforms.

## Results

<p align="center"><img src="DriverMHG_Code/blob/master/utils/results_DriverMHG.jpg" align="middle" width="900" title="Results of Efficient 3DCNNs" /></p>

## Dataset Preparation

### DriverMHG

* Download dataset from its [official website](https://www.ei.tum.de/mmk/drivermhg/).

* Generate annotation files 'all_left.json' and 'all_right.json' according to the README file of the downloaded dataset. 

* Put the 'all_left.json' and 'all_right.json' in the 'annotation_DriverMHG' folder.



## Running the code

Please check all the 'Resource efficient 3D CNN models' in models folder and run the code by providing the necessary parameters. An example run is given as follows:

- Training from scratch:
```bash
python main.py --root_path ~/ \
	--video_path ~/datasets/DriverMHG \
	--annotation_path DriverMHG_Code/annotation_DriverMHG/all_left.json \
	--result_path DriverMHG_Code/results \
	--dataset drivermhg \
	--n_classes 7 \
	--model mobilenetv2 \
	--model_depth 101 \
	--resnet_shortcut B \
	--resnext_cardinality 32 \
	--groups 3 \
	--width_mult 1.0 \
	--train_crop left \
	--learning_rate 0.1 \
	--sample_duration 16 \
	--downsample 3 \
	--batch_size 80 \
	--modality IR \
	--n_threads 16 \
	--checkpoint 1 \
	--n_val_samples 1 \
	--n_epochs 25 \

```

You can directly use run-drivermhg.sh file.


### Augmentations

There are several augmentation techniques available. Please check spatial_transforms.py and temporal_transforms.py for the details of the augmentation methods.

Note: Do not use "RandomHorizontalFlip" for trainings as it alters the class type of some classes (e.g. Swipe_Left --> RandomHorizontalFlip() --> Swipe_Right)



### Calculating FLOPs

In order to calculate FLOPs, run the file 'calculate_FLOP.py'. You need to fist uncomment the desired model in the file. 

## Citation

Please cite the following article if you use this code:

@article{kopuklu2020drivermhg,
  title={DriverMHG: A Multi-Modal Dataset for Dynamic Recognition of Driver Micro Hand Gestures and a Real-Time Recognition Framework},
  author={K{\"o}p{\"u}kl{\"u}, Okan and Ledwon, Thomas and Rong, Yao and Kose, Neslihan and Rigoll, Gerhard},
  journal={arXiv preprint arXiv:2003.00951},
  year={2020}
}

