import os
import torch
from torch.autograd import Variable
import time
import sys
import torch.nn.functional as F
from models import squeezenet
from model import generate_model
from torch import nn
from torch import optim
from model import _modify_first_conv_layer

from utils import AverageMeter, calculate_accuracy


def val_cam(epoch, data_loader, criterion, opt, logger):

    # load RGB Model
    opt.modality = 'RGB'
    model_rgb, parameters = generate_model(opt)
    checkpoint = torch.load( os.path.join('/usr/home/kop/', opt.resume_path))
    model_rgb.load_state_dict(checkpoint['state_dict'])
    model_rgb.cuda()
    model_rgb.eval()

    # load IR Model
    opt.modality = 'IR'
    model_ir, parameters = generate_model(opt)
    checkpoint = torch.load( os.path.join('/usr/home/kop/', opt.resume_path).replace('DriverMHG-RGB', 'DriverMHG-IR').replace('_RGB_', '_IR_'))
    model_ir.load_state_dict(checkpoint['state_dict'])
    model_ir.cuda()
    model_ir.eval()

    # load Depth Model
    opt.modality = 'D'
    model_d, parameters = generate_model(opt)
    checkpoint = torch.load( os.path.join('/usr/home/kop/', opt.resume_path).replace('DriverMHG-RGB', 'DriverMHG-D').replace('_RGB_', '_D_'))
    model_d.load_state_dict(checkpoint['state_dict'])
    model_d.cuda()
    model_d.eval()



    batch_time = AverageMeter()
    data_time = AverageMeter()
    losses = AverageMeter()
    accuracies = AverageMeter()

    end_time = time.time()
    for i, (inputs_ir, inputs_rgb, inputs_depth, targets) in enumerate(data_loader):
      data_time.update(time.time() - end_time)
      if not opt.no_cuda:
        targets = targets.cuda()
      with torch.no_grad():
        inputs_ir = Variable(inputs_ir)
        inputs_rgb = Variable(inputs_rgb)
        inputs_depth = Variable(inputs_depth)
        targets = Variable(targets)

      outputs_ir = model_ir(inputs_ir)
      outputs_rgb = model_rgb(inputs_rgb)
      outputs_depth = model_d(inputs_depth)

      # Get softmax of the output scores
      outputs_ir = F.softmax(outputs_ir, dim = 1)
      outputs_rgb = F.softmax(outputs_rgb, dim = 1)
      outputs_depth = F.softmax(outputs_depth, dim = 1)

      # Take the average of the scores
      output_avg = (outputs_ir + outputs_rgb)/2 ################ CHANGE HERE
      # output_avg = (outputs_ir + outputs_rgb + outputs_depth)/3 ################ CHANGE HERE
      # output_avg = outputs_rgb

      loss = criterion(output_avg, targets)
      prec1, prec5 = calculate_accuracy(output_avg, targets, topk=(1,5))

      losses.update(loss.data, inputs_ir.size(0))
      accuracies.update(prec1, inputs_ir.size(0))

      batch_time.update(time.time() - end_time)
      end_time = time.time()
      
      print('Epoch: [{0}][{1}/{2}]\t'
            'Time {batch_time.val:.5f} ({batch_time.avg:.5f})\t'
            'Data {data_time.val:.5f} ({data_time.avg:.5f})\t'
            'Loss {loss.val:.4f} ({loss.avg:.4f})\t'
            'Acc {acc.val:.3f} ({acc.avg:.3f})'.format(
                epoch,
                i + 1,
                len(data_loader),
                batch_time=batch_time,
                data_time=data_time,
                loss=losses,
                acc=accuracies))

    logger.log({'epoch': epoch,
                'loss': losses.avg.item(),
                'acc': accuracies.avg.item(),
                'acc': accuracies.avg.item()})